/*
 * DPAPRecord factory class
 *
 * Copyright (C) 2008 W. Michael Petullo <mike@flyn.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "util.h"
#include "photo-meta-reader.h"
#include "dmapd-dpap-record.h"
#include "dmapd-dpap-record-factory.h"

static DmapdDPAPRecordFactory *factory_singleton;

struct DmapdDPAPRecordFactoryPrivate {
        PhotoMetaReader *photo_meta_reader;
};

enum {
        PROP_0,
	PROP_META_READER
};

static void
dmapd_dpap_record_factory_set_property (GObject *object,
                                        guint prop_id,
                                        const GValue *value,
                                        GParamSpec *pspec)
{
        DmapdDPAPRecordFactory *factory = DMAPD_DPAP_RECORD_FACTORY (object);
        GObject *obj;

        switch (prop_id) {
                case PROP_META_READER:
                        /* FIXME: should I free first? */
                        factory->priv->photo_meta_reader = PHOTO_META_READER (g_value_get_pointer (value));
                        break;
                default:
                        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                        break;
        }
}

static void
dmapd_dpap_record_factory_get_property (GObject *object,
                                        guint prop_id,
                                        GValue *value,
                                        GParamSpec *pspec)
{
        DmapdDPAPRecordFactory *factory = DMAPD_DPAP_RECORD_FACTORY (object);

        switch (prop_id) {
                case PROP_META_READER:
                        g_value_set_pointer (value, factory->priv->photo_meta_reader);
                        break;
                default:
                        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                        break;
        }
}

DMAPRecord *
dmapd_dpap_record_factory_create  (DMAPRecordFactory *factory, gpointer user_data)
{
	DmapdDPAPRecord *record = NULL;
	GByteArray *blob;

	blob = cache_read ((const char *) user_data);

	if (blob) {
		record = dmapd_dpap_record_new_from_blob (blob);
		g_byte_array_free (blob, TRUE);
	} else {
		size_t size;
		record = dmapd_dpap_record_new ((const char *) user_data, DMAPD_DPAP_RECORD_FACTORY (factory)->priv->photo_meta_reader);
		if (record != NULL) {
			blob = dmapd_dpap_record_to_blob (record, &size);
			cache_store ((const char *) user_data, blob, size);
			g_byte_array_free (blob, TRUE);
		}
	}

	return DMAP_RECORD (record);
}

static void
dmapd_dpap_record_factory_init (DmapdDPAPRecordFactory *factory)
{
	factory->priv = DMAPD_DPAP_RECORD_FACTORY_GET_PRIVATE (factory);
}

static void
dmapd_dpap_record_factory_class_init (DmapdDPAPRecordFactoryClass *klass)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

        g_type_class_add_private (klass, sizeof (DmapdDPAPRecordFactoryPrivate));

        gobject_class->set_property = dmapd_dpap_record_factory_set_property;
        gobject_class->get_property = dmapd_dpap_record_factory_get_property;

        g_object_class_install_property (gobject_class, PROP_META_READER,
                                         g_param_spec_pointer ("meta-reader",
                                                              "Meta Reader",
                                                              "Meta Reader",
                                                              G_PARAM_READWRITE |
                                                              G_PARAM_CONSTRUCT_ONLY));
}

static void
dmapd_dpap_record_factory_interface_init (gpointer iface, gpointer data)
{
	DMAPRecordFactoryInterface *factory = iface;

	g_assert (G_TYPE_FROM_INTERFACE (factory) == TYPE_DMAP_RECORD_FACTORY);

	factory->create = dmapd_dpap_record_factory_create;
}

G_DEFINE_TYPE_WITH_CODE (DmapdDPAPRecordFactory, dmapd_dpap_record_factory, G_TYPE_OBJECT, 
			 G_IMPLEMENT_INTERFACE (TYPE_DMAP_RECORD_FACTORY,
					        dmapd_dpap_record_factory_interface_init))
