/*   FILE: util.c -- utility functions
 * AUTHOR: W. Michael Petullo <mike@flyn.org>
 *   DATE: 01 January 2009 
 *
 * Copyright (c) 2009 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <libdmapsharing/dmap.h>

#include "util.h"

#define CACHEDIR "/var/cache/dmapd"

gchar *
cache_path (const gchar *imagepath)
{
	gchar *cachepath = NULL;
	guchar hash[33] = { 0 };

	dmap_hash_generate (1, (const guchar*) imagepath, 2, hash, 0);
	cachepath = g_strdup_printf (CACHEDIR"/%s", hash);

	return cachepath;
}

GByteArray *
cache_read (const gchar *imagepath)
{
	gchar *data;
	size_t size;
	GByteArray *blob = NULL;
	gchar *cachepath;
	GError *error = NULL;

	cachepath = cache_path (imagepath);

	g_file_get_contents (cachepath, &data, &size, &error);
	if (error != NULL) {
		g_debug ("No record cached at %s", cachepath);
	} else {
		blob = g_byte_array_new ();
		g_byte_array_append (blob, data, size);
		g_free (data);
	}

	g_free (cachepath);

	return blob;
}

void
cache_store (const gchar *imagepath, GByteArray *blob, size_t size)
{
	struct stat st;
	gchar *cachepath;
	GError *error = NULL;
	/* NOTE: g_stat seemed broken; would corrupt GError *error. */
	if (stat (CACHEDIR, &st) != 0) {
		g_error ("cache directory %s does not exist", CACHEDIR);
	}
	if (! (st.st_mode & S_IFDIR)) {
		g_error ("%s is not a directory", CACHEDIR);
	}
	cachepath = cache_path (imagepath);
	g_file_set_contents (cachepath, blob->data, size, &error);
	if (error != NULL) {
		g_error ("Error writing %s", cachepath);
	}

	g_free (cachepath);
}
